#include <srutil/event.hpp>
#include <srutil/delegate.hpp>
#include <boost/bind.hpp>
#include <boost/noncopyable.hpp>
#include <iostream>
using namespace srutil;

typedef delegate1<void, int> MyDelegate;

class EventProducer
{
public:
	event_source<MyDelegate> const& getEventSource() const {return source;}

	void test()
	{
		source.emit(MyDelegate::invoker_type(5));
	}

private:
	event_source<MyDelegate> source;
};

class FastEventConsumer : private boost::noncopyable
{
public:
	FastEventConsumer(EventProducer const * producer)
	{
		binder.bind(producer->getEventSource(),
			MyDelegate::from_method<FastEventConsumer, &FastEventConsumer::method>(this));
	}

private:
	event_binder<MyDelegate> binder;
	void method(int)
	{
		std::cout << "FastEventConsumer has received event.\n";
	}
};

class FlexEventConsumer : private boost::noncopyable
{
public:
	FlexEventConsumer(EventProducer const * producer)
	{
		binder.bind(producer->getEventSource(),
			boost::bind(&FlexEventConsumer::method, this, _1, 10));
	}

private:
	event_flex_binder<MyDelegate> binder;

	void method(int, int)
	{
		std::cout << "FlexEventConsumer has received event.\n";
	}
};

int main()
{
	EventProducer producer;
	FastEventConsumer fast_consumer(&producer);
	FlexEventConsumer flex_consumer(&producer);

	producer.test();

	return 0;
}
